SystemOrganization addCategory: #'Tweak-Projects'!

Project subclass: #CTweakProjectInMorphic
	instanceVariableNames: 'uiProcess island done'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CTweakProjectInMorphic class methodsFor: 'class initialization' stamp: 'ar 6/18/2005 16:45'!
initialize
	"CTweakProjectInMorphic initialize"
	TheWorldMenu unregisterOpenCommandWithReceiver: self.
	TheWorldMenu registerOpenCommand: 
		{ 'Tweak Project' . {CTweakProjectInMorphic . #open }. 'A Tweak Project' }.
! !

!CTweakProjectInMorphic class methodsFor: 'accessing' stamp: 'ar 6/18/2005 14:51'!
new
	| new |
	new := self basicNew.
	self addingProject: new.
	new initTweak.
	^new! !

!CTweakProjectInMorphic class methodsFor: 'accessing' stamp: 'ar 6/18/2005 14:51'!
open
	(ProjectViewMorph on: self new) openInWorld! !

!CTweakProjectInMorphic methodsFor: 'running' stamp: 'ar 6/18/2005 15:28'!
checkForNewScreenSize
	Display extent = DisplayScreen actualScreenSize ifTrue:[^self].
	DisplayScreen startUp.
	island project canvas: (CTransformCanvas on: Display).
	island project world extent: Display extent.! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 14:58'!
enter: returningFlag revert: revertFlag saveForRevert: saveForRevert
	"Install my ChangeSet, Transcript, and scheduled views as current globals. If returningFlag is true, we will return to the project from whence the current project was entered; don't change its previousProject link in this case."
	(revertFlag or:[saveForRevert]) ifTrue:[^self error: 'Won''t do this'].
	self isCurrentProject ifTrue: [^ self].
	"Check the guards"
	guards ifNotNil:
		[guards := guards reject: [:obj | obj isNil].
		guards do: [:obj | obj okayToEnterProject ifFalse: [^ self]]].
	CurrentProject finalExitActions.
	ProjectHistory remember: CurrentProject.

	"Update the display depth and make a thumbnail of the current project"
	CurrentProject displayDepth: Display depth.

	"Show the project transition.
	Note: The project zoom is run in the context of the old project,
		so that eventual errors can be handled accordingly"
	displayDepth == nil ifTrue: [displayDepth := Display depth].
	self installNewDisplay: Display extent depth: displayDepth.
	(self showZoom) ifTrue: [
		self displayZoom: CurrentProject parent ~~ self].
	Smalltalk at: #ScorePlayer ifPresentAndInMemory:[:playerClass | 
		playerClass allSubInstancesDo:[:player | player pause]].
	returningFlag
			ifTrue: [nextProject := CurrentProject]
			ifFalse: [previousProject := CurrentProject].
	CurrentProject saveState.
	CurrentProject := self.
	self installProjectPreferences.
	ChangeSet  newChanges: changeSet.
	TranscriptStream newTranscript: transcript.
	Sensor flushKeyboard.
	Smalltalk isMorphic ifTrue: [World pauseEventRecorder].
	ProjectHistory remember: CurrentProject.
	self finalEnterActions.
	done := false.
	uiProcess := [self run] forkAt: Processor userSchedulingPriority.
	Processor terminateActive.! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 14:51'!
finalEnterActions
	"Perform the final actions necessary as the receiver project is entered"
	World := nil.
	Sensor installInterruptWatcher.
	Smalltalk installLowSpaceWatcher.
	UIManager default: TweakUIManager new.! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 15:04'!
finalExitActions
	done := true.
	island project canvas: nil.
	UIManager default: nil.
	super finalExitActions.
! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 16:23'!
imageFormOfSize: extentPoint depth: d
	"Don't ever think of returning anything but a DisplayScreen - it breaks project transitions."
	| display canvas |
	display := DisplayScreen extent: extentPoint depth: d.
	display fillWhite.
	canvas := CTransformCanvas on: display.
	island project world asPrimCostume fullDrawOn: canvas.
	^display! !

!CTweakProjectInMorphic methodsFor: 'initialize' stamp: 'ar 6/21/2010 11:23'!
initTweak
	changeSet := ChangeSet new.
	transcript := Smalltalk at: #CTranscriptStream ifPresent:[:cls| cls new].
	displayDepth := Display depth.
	parentProject := CurrentProject.

	island := CProjectIsland new.
	island scheduler: (island new: ScriptScheduler).
	island project: (island new: (Smalltalk at: #EToysProject)).
	island project newProjectWith: (island new: (Smalltalk at: #CProjectBuilder)).
	island project canvas: (CTransformCanvas on: Display).
	island project world extent: Display extent.
	island scheduler run.

	island project startScript: #run.
	island project primaryHand startScript: #processEventsLoop.
! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 15:53'!
leaveTo: aProject
	"Leave this project entering aProject"
	Processor activeProcess isScriptingProcess ifTrue:[
		"Cannot do this from the inside"
		[self leaveTo: aProject] fork.
		^self].
	done := true.
	aProject enter.! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 7/12/2007 17:59'!
makeThumbnail
	"Make a thumbnail image of this project from the Display."
	viewSize ifNil: [viewSize := Display extent // 8].
	thumbnail := Form extent: viewSize depth: Display depth.
	(WarpBlt current toForm: thumbnail)
			sourceForm: Display;
			cellSize: 2;  "installs a colormap"
			combinationRule: Form over;
			copyQuad: (Display boundingBox) innerCorners
			toRect: (0@0 extent: viewSize).
	Smalltalk at: #InternalThreadNavigationMorph 
		ifPresent:[:itnm|  itnm cacheThumbnailFor: self].
	^thumbnail
! !

!CTweakProjectInMorphic methodsFor: 'running' stamp: 'ar 6/18/2005 15:29'!
restart
	"restart the hard way"
	uiProcess ifNotNil:[uiProcess terminate].
	uiProcess := [self run] forkAt: Processor userSchedulingPriority.! !

!CTweakProjectInMorphic methodsFor: 'running' stamp: 'ar 6/18/2005 16:32'!
run
	island project canvas: (CTransformCanvas on: Display).
	island project world extent: Display extent.
	[done] whileFalse:[
		self checkForNewScreenSize.
		island scheduler run
	].
	uiProcess == Processor activeProcess ifTrue:[uiProcess := nil].
! !

!CTweakProjectInMorphic methodsFor: 'entering' stamp: 'ar 6/18/2005 14:51'!
saveState
	"Save the current state in me prior to leaving this project"
	changeSet := ChangeSet current.
	thumbnail ifNotNil: [thumbnail hibernate].
	Sensor flushAllButDandDEvents. "Will be reinstalled by World>>install"
	transcript := Transcript.
	uiProcess := nil.! !

!Integer methodsFor: '*Tweak-Projects' stamp: 'ar 5/25/2005 16:29'!
asByteArray
	"Convert the receiver into a byte array"
	| ba |
	ba := ByteArray new: (self digitLength max: 4).
	1 to: ba size do:[:i| ba at: i put: (self digitAt: i)].
	^ba! !

IslandReader subclass: #CProjectReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectReader methodsFor: 'reading' stamp: 'ar 6/16/2005 20:36'!
readProjectFrom: aStream
	| versionID archive data roots |
	versionID := UUID nilUUID.
	aStream next: 16 into: versionID.
	(versionID = CProjectWriter archiveVersionID) 
		ifFalse:[^self error: 'Not a project file'].
	archive := ZipArchive new.
	archive readFrom: aStream.
	data := (archive memberNamed: 'project.isg') byteStream contents.
	dataStream := IslandStream with: data.
	dataStream reset.
	(dataStream nextUUID = CProjectWriter projectVersionID) 
		ifFalse:[^self error: 'Not a project file'].
	imports := self importNext.
	segment := self importNext.
	roots := self loadSegmentFrom: segment outPointers: imports.
	^roots at: 1! !

!CProjectReader methodsFor: 'reading' stamp: 'ar 6/16/2005 20:36'!
swapInProjectFrom: aStream
	| roots data |
	data := ByteArray new: aStream size.
	aStream next: data size into: data startingAt: 1.
	dataStream := IslandStream with: data.
	dataStream reset.
	(dataStream nextUUID = CProjectWriter projectVersionID) 
		ifFalse:[^self error: 'Not a project file'].
	imports := self importNext.
	segment := self importNext.
	roots := self loadSegmentFrom: segment outPointers: imports.
	^roots at: 1! !

CObject subclass: #CProject2
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProject2 commentStamp: 'ar 4/26/2005 20:42' prior: 0!
CProject bundles all the stuff we need to run a project.

Fields:
	eventQueue	<SharedQueue>	The event queue for (local) events.
	canvas		<CCanvas>		The canvas for (local) drawing operations.
	window		<...>				If present, a reference to the window hosting the project.

	world	<CPlayer>			The world of this project.
	hands	<Array of: Hand>	The hands active in this project.
	scheduler <ScriptScheduler>	The script scheduler for this project.
!

CProject2 defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="eventQueue" toSet="eventQueue:" changeEvent="eventQueueChanged">eventQueue</field>
	<field toGet="messageQueue" toSet="messageQueue:" changeEvent="messageQueueChanged">messageQueue</field>
	<field toGet="canvas" toSet="canvas:" changeEvent="canvasChanged">canvas</field>
	<field toGet="window" toSet="window:" changeEvent="windowChanged">window</field>
	<field toGet="world" toSet="world:" changeEvent="worldChanged">world</field>
	<field toGet="hands" toSet="hands:" changeEvent="handsChanged">hands</field>
	<field toGet="primaryHand" toSet="primaryHand:" changeEvent="primaryHandChanged">primaryHand</field>
	<field toGet="maxLayoutRounds" toSet="maxLayoutRounds:" changeEvent="maxLayoutRoundsChanged">maxLayoutRounds</field>
	<field toGet="frameNumber" toSet="frameNumber:" changeEvent="frameNumberChanged">frameNumber</field>
	<field toGet="killFrame" toSet="killFrame:" changeEvent="killFrameChanged">killFrame</field>
	<field toGet="displayHook" toSet="displayHook:" changeEvent="displayHookChanged">displayHook</field>
	<field toGet="topWindow" toSet="topWindow:" changeEvent="topWindowChanged">topWindow</field>
	<field toGet="fileName" toSet="fileName:" changeEvent="fileNameChanged">fileName</field>
	<field toGet="lastFileDirectory" toSet="lastFileDirectory:" changeEvent="lastFileDirectoryChanged">lastFileDirectory</field>
</fields>'!

!CProject2 class methodsFor: 'accessing' stamp: 'ar 7/8/2007 13:34'!
defaultProject
	"Answer the default project class"
	^Default ifNil:[self]! !

!CProject2 class methodsFor: 'accessing' stamp: 'ar 7/8/2007 13:35'!
defaultProject: aClassOrNil
	"Set the default project class"
	Default := aClassOrNil! !

!CProject2 methodsFor: 'hands' stamp: 'ar 5/22/2005 22:43'!
addHand: aHand
	"@@@@: Fix this. Needs to handle #outOfWorld etc. :@@@@"
	aHand container ifNotNil:[aHand container removeHand: aHand].
	aHand asPrimCostume setContainer: world asPrimCostume.
	hands := (hands copyWithout: aHand) copyWith: aHand.
! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:14'!
canvas
	"Answer the canvas of the receiver"
	<bewareOf: #canvasChanged>
	^self propertyValueAt: #canvas! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:14'!
canvas: aValue
	"Modify the receiver's canvas"
	^self propertyValueAt: #canvas put: aValue with: #canvasChanged! !

!CProject2 methodsFor: 'initialize' stamp: 'ar 2/9/2006 11:22'!
destroy
	world ifNotNil:[
		world destroy.
		world := nil.
	].
	super destroy.! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 2/9/2006 12:03'!
displayHook
	"Answer the displayHook of the receiver"
	<bewareOf: #displayHookChanged>
	^self propertyValueAt: #displayHook! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 2/9/2006 12:03'!
displayHook: aValue
	"Modify the receiver's displayHook"
	^self propertyValueAt: #displayHook put: aValue with: #displayHookChanged! !

!CProject2 methodsFor: 'drawing' stamp: 'bgf 10/10/2007 16:25'!
drawOn: aCanvas
	| |
	aCanvas ifNil:[^self].
	killFrame == true ifTrue:[
		"Suppress current frame"
		killFrame := false.
		^#()].

	frameNumber := frameNumber + 1 bitAnd: SmallInteger maxVal.
	^ displayHook 
		ifNotNil:[ displayHook drawWorldOn: aCanvas]
		ifNil: [ self drawWorldOn: aCanvas ].
! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 3/30/2007 18:59'!
drawOn: aCanvas in: rectToFill
	world asPrimCostume fullDrawOn: aCanvas.
	hands do:[:h| 
		(h visible not or:[h showHardwareCursor and:[h mouseCursor isCursor]]) ifFalse:[
			aCanvas transformBy: world asPrimCostume parentToChild during:[:cc|
				h asPrimCostume fullDrawOn: cc.
			].
		].
	].! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 2/6/2009 17:53'!
drawWorldOn: aCanvas

	"Draw the project content.  This is the drawing process used when no drawing hook is installed;
	 it may also be called as a fallback from within a drawing-hook's drawWorldOn: method"

	| validList rectToFill flashList recorder dirtyRects worldCostume |
	
	worldCostume := world asPrimCostume.
	aCanvas deferUpdates: true.
	recorder := worldCostume localCache damageRecorder.
	dirtyRects := recorder invalidRectsFullBounds: worldCostume bounds.

	"sort by areas to draw largest portions first"
	dirtyRects := dirtyRects asArray sort:[:r1 :r2| r1 area > r2 area].
	recorder reset.
	validList := OrderedCollection new: dirtyRects size.
	rectToFill := nil.
	dirtyRects do: [:dirtyRect |
		dirtyRect allAreasOutsideList: validList do:[:r| 
			rectToFill 
				ifNil:[rectToFill := r copy]
				ifNotNil:[rectToFill := rectToFill quickMerge: r].
			validList add: r]].
	rectToFill ifNotNil:[
		aCanvas pushState; clipBy: rectToFill.

		"clear background of the worldCostume"
		validList do:[:r| (worldCostume areasRemainingToFill: r) 
			do:[:r2| aCanvas clearRectangle: r2]].

		worldCostume topDownFullDrawOn: aCanvas damageList: validList.
		hands do:[:h| 
			(h visible not or:[h showHardwareCursor and:[h mouseCursor isCursor]]) ifFalse:[
				h asPrimCostume fullDrawOn: aCanvas]].
		aCanvas popState.
	].
	aCanvas finish.
	flashList := validList.
	aCanvas forceDamageToScreen: validList.
	false ifTrue:[aCanvas flashRects: flashList color: Color cyan].
	aCanvas deferUpdates: false.
	"Sender might need the damage rectangles"
	^validList! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:13'!
eventQueue
	"Answer the eventQueue of the receiver"
	<bewareOf: #eventQueueChanged>
	^self propertyValueAt: #eventQueue! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:13'!
eventQueue: aValue
	"Modify the receiver's eventQueue"
	^self propertyValueAt: #eventQueue put: aValue with: #eventQueueChanged! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:01'!
fileName
	"Answer the fileName of the receiver"
	<bewareOf: #fileNameChanged>
	^self propertyValueAt: #fileName! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:01'!
fileName: aValue
	"Modify the receiver's fileName"
	^self propertyValueAt: #fileName put: aValue with: #fileNameChanged! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/27/2005 15:02'!
frameNumber
	"Answer the frameNumber of the receiver"
	<bewareOf: #frameNumberChanged>
	^self propertyValueAt: #frameNumber! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/27/2005 15:02'!
frameNumber: aValue
	"Modify the receiver's frameNumber"
	^self propertyValueAt: #frameNumber put: aValue with: #frameNumberChanged! !

!CProject2 methodsFor: 'window' stamp: 'ar 4/26/2005 20:24'!
fullScreenMode
	"Only the window can handle fullscreen requests"
	window ifNil:[^false].
	^window fullScreenMode! !

!CProject2 methodsFor: 'window' stamp: 'ar 4/26/2005 20:24'!
fullScreenMode: aBool
	"Only the window can handle fullscreen requests"
	window ifNil:[^false].
	^window fullScreenMode: aBool! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:12'!
hands
	"Answer the hands of the receiver"
	<bewareOf: #handsChanged>
	^self propertyValueAt: #hands! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:12'!
hands: aValue
	"Modify the receiver's hands"
	^self propertyValueAt: #hands put: aValue with: #handsChanged! !

!CProject2 methodsFor: 'initialize' stamp: 'ar 4/27/2005 15:44'!
initialize
	"Note: For creating a project you must provide world and hands explicitly."
	super initialize.
	hands := #().
	maxLayoutRounds := 3.
	frameNumber := 0.
	killFrame := false.! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:34'!
killFrame
	"Answer the killFrame of the receiver"
	<bewareOf: #killFrameChanged>
	^self propertyValueAt: #killFrame! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:34'!
killFrame: aValue
	"Modify the receiver's killFrame"
	^self propertyValueAt: #killFrame put: aValue with: #killFrameChanged! !

!CProject2 methodsFor: 'window' stamp: 'ar 4/26/2005 20:25'!
label
	window ifNil:[^''].
	^window label! !

!CProject2 methodsFor: 'window' stamp: 'ar 4/26/2005 20:25'!
label: aString
	"Label the window"
	window ifNotNil:[window label: aString].! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 6/5/2005 12:10'!
lastFileDirectory
	"Answer the lastFileDirectory of the receiver"
	<bewareOf: #lastFileDirectoryChanged>
	^self propertyValueAt: #lastFileDirectory! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 6/5/2005 12:10'!
lastFileDirectory: aValue
	"Modify the receiver's lastFileDirectory"
	^self propertyValueAt: #lastFileDirectory put: aValue with: #lastFileDirectoryChanged! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:02'!
maxLayoutRounds
	"Answer the maxLayoutRounds of the receiver"
	<bewareOf: #maxLayoutRoundsChanged>
	^self propertyValueAt: #maxLayoutRounds! !

!CProject2 methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:02'!
maxLayoutRounds: aValue
	"Modify the receiver's maxLayoutRounds"
	^self propertyValueAt: #maxLayoutRounds put: aValue with: #maxLayoutRoundsChanged! !

!CProject2 methodsFor: 'accessing' stamp: 'das 11/29/2009 13:41'!
messageQueue
	"Answer the messageQueue of the receiver"
	<bewareOf: #messageQueueChanged>
	^self propertyValueAt: #messageQueue! !

!CProject2 methodsFor: 'accessing' stamp: 'das 11/29/2009 13:41'!
messageQueue: aValue
	"Modify the receiver's messageQueue"
	^self propertyValueAt: #messageQueue put: aValue with: #messageQueueChanged! !

!CProject2 methodsFor: 'running' stamp: 'ar 4/26/2005 20:26'!
minCycleTime
	"Answer the minimal time one world cycle should take. If the current cycle is faster then we may give the host OS a few cycles back"
	^Preferences higherPerformance
		ifTrue: [0]
		ifFalse: [20]
! !

!CProject2 methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:08'!
newProject
	^self newProjectWith: CPlayer new.! !

!CProject2 methodsFor: 'initialize' stamp: 'ar 5/26/2005 14:08'!
newProject: worldMaker
	"Provide a world and hands"
	self newProjectWith: worldMaker new.! !

!CProject2 methodsFor: 'initialize' stamp: 'ar 6/28/2005 18:24'!
newProjectWith: aWorld
	"Provide a world and hands"
	eventQueue := SharedQueue new.
	world := aWorld.
	world asPrimCostume isWorld: true.
	world asPrimCostume localCache: CProjectCache new.
	world asPrimCostume intoWorld: world. "trigger initial scripts"
	self addHand: (primaryHand := CHandPlayer new).
	primaryHand showHardwareCursor: true.! !

!CProject2 methodsFor: 'events' stamp: 'ar 5/24/2005 17:39'!
onShutDownProject: quitting
	"Shut down this project"
	<on: shutDownProject>
	world ifNotNil:[world asPrimCostume shutDownProject: quitting].! !

!CProject2 methodsFor: 'events' stamp: 'ar 5/26/2005 22:06'!
onStartUpProject: resuming
	"Start up this project"
	<on: startUpProject>
	world ifNotNil:[
		world asPrimCostume startUpProject: resuming.
		world invalidate.
	].! !

!CProject2 methodsFor: 'top window' stamp: 'ar 5/11/2005 14:39'!
onTopWindowChanged: newWindow from: oldWindow
	"update top window"
	<on: topWindowChanged>
	oldWindow ifNotNil:[oldWindow signal: #passivate].
	newWindow ifNotNil:[newWindow signal: #activate].
! !

!CProject2 methodsFor: 'top window' stamp: 'ar 9/30/2005 16:07'!
onTopWindowClosed
	| list |
	<on: close in: topWindow>
	world ifNil:[^self].
	self topWindow: nil.
	list := CWindowManager default windowsIn: world satisfying:[:w| w isCollapsed not].
	list size = 0 ifFalse:[list last beTopWindow].! !

!CProject2 methodsFor: 'events' stamp: 'ar 4/20/2007 19:25'!
onWorldChanged
	<on: worldChanged>
	world ifNotNil:[
		world asPrimCostume isWorld: true.
		world asPrimCostume localCache: CProjectCache new.
	].! !

!CProject2 methodsFor: 'hands' stamp: 'ar 5/28/2005 15:06'!
primaryHand
	"Answer the primaryHand of the receiver"
	<bewareOf: #primaryHandChanged>
	^self propertyValueAt: #primaryHand! !

!CProject2 methodsFor: 'hands' stamp: 'ar 5/28/2005 15:06'!
primaryHand: aValue
	"Modify the receiver's primaryHand"
	^self propertyValueAt: #primaryHand put: aValue with: #primaryHandChanged! !

!CProject2 methodsFor: 'running' stamp: 'das 11/29/2009 17:02'!
processPyMessage: message

	"place holder"! !

!CProject2 methodsFor: 'hands' stamp: 'ar 4/27/2005 15:46'!
removeHand: aHand
	"@@@@: Fix this. Needs to handle #outOfWorld etc. :@@@@"
	(hands includes: aHand) ifFalse: [^self].
	aHand asPrimCostume setContainer: nil.
	aHand project: nil.
	aHand invalidate.
	hands := hands copyWithout: aHand.! !

!CProject2 methodsFor: 'running' stamp: 'ar 6/28/2005 19:47'!
run
	| time |
	[true] whileTrue:[
		time := Time millisecondsToRun:[
			self runOneCycle.
			self drawOn: canvas.
			self setHardwareCursor.
		].
		self minCycleTime > 0
			ifTrue: [(Delay forMilliseconds: (self minCycleTime - time max: 1)) wait]
			ifFalse: [Processor yield].
	].! !

!CProject2 methodsFor: 'running' stamp: 'das 11/29/2009 19:01'!
runOneCycle
	"Run a world cycle."
	| script event layoutMax layoutRepeat message |
	"Signal the start of a new world cycle"
	self signalTick: #tick.
	self syncScript.

	"Process events from eventQueue"
	[event := eventQueue nextOrNil.
	event == nil] whileFalse:[
		script := primaryHand startScript: #dispatchEvent: withArguments:{event clone}.
		script hand: primaryHand.
		self syncScript.
	].

	"Process messages from messageQueue"
	messageQueue ifNotNil:[
		[message := messageQueue nextOrNil.
		message == nil] whileFalse:[
			world processMessage: message.
		].
	].

	primaryHand signal: #updateMouseOver.
	self syncScript.

	"compute updated screen layout"
	layoutMax := maxLayoutRounds + (frameNumber bitAnd: 1).
	layoutRepeat := 0.
	[world layoutNeeded and:[
		(layoutRepeat := layoutRepeat + 1) <= layoutMax]] whileTrue:[
		script := world startScript: #fullBounds.
		script hand: primaryHand.
		self syncScript.
	].
! !

!CProject2 methodsFor: 'running' stamp: 'ar 4/19/2007 18:25'!
setHardwareCursor
	"set the new hardware cursor"
	| cursor |
	primaryHand visible ifFalse:[^nil].
	cursor := primaryHand mouseCursor.
	cursor isCursor ifFalse:[cursor := Cursor blank].
	cursor == Sensor currentCursor ifFalse:[Sensor currentCursor: cursor].

! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 5/11/2005 17:28'!
topWindow
	"Answer the topWindow of the receiver"
	<bewareOf: #topWindowChanged>
	^self propertyValueAt: #topWindow! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 5/11/2005 17:28'!
topWindow: aValue
	"Modify the receiver's topWindow"
	^self propertyValueAt: #topWindow put: aValue with: #topWindowChanged! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:13'!
window
	"Answer the window of the receiver"
	<bewareOf: #windowChanged>
	^self propertyValueAt: #window! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:13'!
window: aValue
	"Modify the receiver's window"
	^self propertyValueAt: #window put: aValue with: #windowChanged! !

!CProject2 methodsFor: 'top window' stamp: 'ar 5/24/2005 20:38'!
windowManager
	"Answer the project's window manager"
	^self! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:14'!
world
	"Answer the world of the receiver"
	<bewareOf: #worldChanged>
	^self propertyValueAt: #world! !

!CProject2 methodsFor: 'accessing' stamp: 'ar 4/26/2005 20:14'!
world: aValue
	"Modify the receiver's world"
	^self propertyValueAt: #world put: aValue with: #worldChanged! !

!MultiByteBinaryOrTextStream methodsFor: '*Tweak-Projects' stamp: 'ar 5/25/2005 23:15'!
next: anInteger putAll: aCollection startingAt: startIndex
	^super next: anInteger putAll: aCollection asString startingAt: startIndex! !

CEventBehavior subclass: #CDragMaker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

CDragMaker defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="maker" toSet="maker:" changeEvent="makerChanged">maker</field>
	<field toGet="proto" toSet="proto:" changeEvent="protoChanged">proto</field>
	<field toGet="msg" toSet="msg:" changeEvent="msgChanged">msg</field>
</fields>'!

!CDragMaker methodsFor: 'accessing' stamp: 'ar 2/7/2005 16:25'!
maker
	"Answer the maker of the receiver"
	<bewareOf: #makerChanged>
	^self propertyValueAt: #maker! !

!CDragMaker methodsFor: 'accessing' stamp: 'ar 2/7/2005 16:25'!
maker: aValue
	"Modify the receiver's maker"
	^self propertyValueAt: #maker put: aValue with: #makerChanged! !

!CDragMaker methodsFor: 'events' stamp: 'ar 2/7/2005 16:52'!
mouseCursor
	^Cursor crossHair asForm! !

!CDragMaker methodsFor: 'accessing' stamp: 'ar 5/21/2005 12:56'!
msg
	"Answer the msg of the receiver"
	<bewareOf: #msgChanged>
	^self propertyValueAt: #msg! !

!CDragMaker methodsFor: 'accessing' stamp: 'ar 5/21/2005 12:56'!
msg: aValue
	"Modify the receiver's msg"
	^self propertyValueAt: #msg put: aValue with: #msgChanged! !

!CDragMaker methodsFor: 'events' stamp: 'ar 5/21/2005 12:58'!
newProto
	msg ifNil:[msg := #new].
	proto := maker perform: msg.
	proto locked: true.! !

!CDragMaker methodsFor: 'events' stamp: 'ar 7/14/2005 19:48'!
onMouseDown
	| firstPoint nextPoint ext |
	<on: mouseDown in: target>
	proto ifNil:[self newProto].
	ext := proto extent.
	firstPoint := target cursorPoint.
	target add: proto.
	self pauseScript: #onMouseMove.
	target hand forEach: #mouseMove do:[
		nextPoint := target cursorPoint.
		proto bounds: (firstPoint rect: nextPoint).
	] until: #mouseUp.
	proto close.
	(proto width < 10 and:[proto height < 10]) ifTrue:[proto extent: ext].
	target makeNew: self in: proto bounds.
	proto := nil.
	target releaseTool: self.! !

!CDragMaker methodsFor: 'events' stamp: 'ar 5/21/2005 12:58'!
onMouseEnter
	<on: mouseEnter in: target>
	proto ifNil:[self newProto].
	proto topLeft: target cursorPoint.
	target add: proto.! !

!CDragMaker methodsFor: 'events' stamp: 'ar 5/21/2005 12:59'!
onMouseLeave
	<on: mouseLeave in: target>
	proto ifNotNil:[proto close].! !

!CDragMaker methodsFor: 'events' stamp: 'ar 5/21/2005 12:59'!
onMouseMove
	<on: mouseMove in: target>
	proto ifNil:[self newProto].
	proto topLeft: target cursorPoint.
	(proto isIn: target) ifFalse:[target add: proto].! !

!CDragMaker methodsFor: 'accessing' stamp: 'ar 2/7/2005 16:37'!
proto
	"Answer the proto of the receiver"
	<bewareOf: #protoChanged>
	^self propertyValueAt: #proto! !

!CDragMaker methodsFor: 'accessing' stamp: 'ar 2/7/2005 16:37'!
proto: aValue
	"Modify the receiver's proto"
	^self propertyValueAt: #proto put: aValue with: #protoChanged! !

CItemWidget subclass: #CProjectIslandCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectIslandCostume commentStamp: '<historical>' prior: 0!
A simple costume that can be used to visualize a project in CProjectBuilder.!

CProjectIslandCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="myIsland" toSet="myIsland:" changeEvent="myIslandChanged">myIsland</field>
	<field toGet="myProject" toSet="myProject:" changeEvent="myProjectChanged">myProject</field>
	<field toGet="myWorld" toSet="myWorld:" changeEvent="myWorldChanged">myWorld</field>
	<field toGet="label" toSet="label:" changeEvent="labelChanged">label</field>
	<field toGet="image" toSet="image:" changeEvent="imageChanged">image</field>
</fields>'!

!CProjectIslandCostume methodsFor: 'drawing' stamp: 'ar 8/5/2005 20:28'!
drawOn: aCanvas in: drawingBounds
	image graphicForm ifNil:[
		image graphicForm: (myWorld asPrimCostume asFormExtent: 128@128).
	].
	^super drawOn: aCanvas in: drawingBounds! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 11:58'!
image
	"Answer the image of the receiver"
	<bewareOf: #imageChanged>
	^self propertyValueAt: #image! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 11:58'!
image: aValue
	"Modify the receiver's image"
	^self propertyValueAt: #image put: aValue with: #imageChanged! !

!CProjectIslandCostume methodsFor: 'initialize' stamp: 'ar 8/5/2005 12:28'!
initialize
	super initialize.
	self define: #image as: CPlayer new.
	self define: #label as: CLabel new.
	self pauseScript: #onThumbnailChanged.! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 11:58'!
label
	"Answer the label of the receiver"
	<bewareOf: #labelChanged>
	^self propertyValueAt: #label! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 11:58'!
label: aValue
	"Modify the receiver's label"
	^self propertyValueAt: #label put: aValue with: #labelChanged! !

!CProjectIslandCostume methodsFor: 'private' stamp: 'ar 8/5/2005 12:01'!
listItemValue: aValue
	"See comment in listItemValue:"
	^myIsland := aValue.! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:00'!
myIsland
	"Answer the myIsland of the receiver"
	<bewareOf: #myIslandChanged>
	^self propertyValueAt: #myIsland! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:00'!
myIsland: aValue
	"Modify the receiver's myIsland"
	^self propertyValueAt: #myIsland put: aValue with: #myIslandChanged! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:00'!
myProject
	"Answer the myProject of the receiver"
	<bewareOf: #myProjectChanged>
	^self propertyValueAt: #myProject! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:00'!
myProject: aValue
	"Modify the receiver's myProject"
	^self propertyValueAt: #myProject put: aValue with: #myProjectChanged! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:02'!
myWorld
	"Answer the myWorld of the receiver"
	<bewareOf: #myWorldChanged>
	^self propertyValueAt: #myWorld! !

!CProjectIslandCostume methodsFor: 'accessing' stamp: 'ar 8/5/2005 12:02'!
myWorld: aValue
	"Modify the receiver's myWorld"
	^self propertyValueAt: #myWorld put: aValue with: #myWorldChanged! !

!CProjectIslandCostume methodsFor: 'private' stamp: 'ar 12/11/2005 00:17'!
onInvalidate
	"Whenever we see an invalidation we dump the original graphic and recreate it when the costume is next redrawn. This has the neat side-effect that if the costume isn't visible, we won't ever recreate the image."
	self flag: #fixMe.
	"<on: invalidate in: myWorld> -- this breaks publishing right now"
	image graphic: nil.
	image invalidate.! !

!CProjectIslandCostume methodsFor: 'private' stamp: 'ar 8/5/2005 12:31'!
onIslandChanged
	<on: myIslandChanged>
	myProject := myIsland project.
	myWorld := myProject world.
	self onNameChanged.
	self onInvalidate.! !

!CProjectIslandCostume methodsFor: 'private' stamp: 'ar 12/11/2005 00:18'!
onNameChanged
	self flag: #fixMe.
	"<on: nameChanged in: myProject> -- this breaks publishing right now"
	label value: myProject name.
	balloonHelp := 'Click here to enter ', myProject name storeString.! !

!CProjectIslandCostume methodsFor: 'initialize' stamp: 'ar 8/5/2005 20:28'!
setupCostume
	super setupCostume.
	self userDraw: true.
	self add: image.
	self add: label.
	image graphicEnabled: true.
	image graphicFit: #scale.
	image clipping: true.
	self layout: CTableLayout new.
	self color: (Color gray: 0.9).
	self borderStyle: #simple.
	self borderWidth: 1.
	self borderColor: Color black.
	self roundCorners: true.
	self extent: 80@60.! !

BorderedMorph subclass: #CProjectMorph
	instanceVariableNames: 'display window island project lastMouseEvent myEventMap noRedraw'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 22:21'!
fileReaderServicesForFile: fullName suffix: suffix

	^(suffix = 'prj') | (suffix = '*') 
		ifTrue: [ Array with: self serviceLoadProject]
		ifFalse: [#()]
! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 6/18/2005 16:46'!
initialize
	"CProjectMorph initialize"
	CTweakProjectInMorphic initialize.

	TheWorldMenu unregisterOpenCommandWithReceiver: self.
	TheWorldMenu registerOpenCommand: 
		{ 'Tweak Project Window' . { self . #openProjectBuilder }. 'A Tweak project window' }.
	FileList registerFileReader: self.! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 21:53'!
open
	^self open: CPlayer! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 22:03'!
open: worldMaker
	"CProjectMorph open: CClassBrowser"
	| root window |
	root := self new.
	window := SystemWindow labelled: worldMaker name.
	window model: root.
	window addMorph: root frame: (0@0 corner: 1@1).
	root window: window.
	root newProject: worldMaker.
	window openInWorld.
	^window! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 23:01'!
openClassBrowser
	^self open: (Smalltalk at: #CClassBrowser ifAbsent:[^self inform: 'CClassBrowser is missing']).! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 6/18/2005 16:43'!
openProjectBuilder
	^self open: (Smalltalk at: #CProjectBuilder ifAbsent:[^self inform: 'CProjectBuilder is missing'])! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 23:15'!
openProjectNamed: fileName
	| root window |
	root := self new.
	window := SystemWindow labelled: 'Project'.
	window model: root.
	window addMorph: root frame: (0@0 corner: 1@1).
	root window: window.
	root openProjectNamed: fileName.
	window openInWorld.
	^window! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 7/12/2007 18:03'!
runAsApp: worldMaker
	"Run this project morph as a full screen standalone app"
	"CProjectMorph runAsApp: MTInterface"
	| root seed |
	worldMaker prepareToRunAsApp. "set image version etc."
	"Flush MC caches"
	MCFileBasedRepository flushAllCaches.
	WorldState addDeferredUIMessage:[
	(self confirm: 'Delete all Morphs?') ifTrue:[
		World submorphs do:[:each| each delete].
	].
	(self confirm: 'Disconnect changes and sources file?') ifTrue:[
		Preferences disable: #warnIfNoSourcesFile.
		Preferences disable: #warnIfNoChangesFile.
	].
	(self confirm: 'Save the image?') ifTrue:[
		AutoStart deinstall. "no autostart please"
		Smalltalk saveAs.
		"Eeek. Reseed islands manually (this should happen automatically!!!!!!)"
		Island allSubInstancesDo:[:each|
			seed := ByteArray new: 128.
			Smalltalk at: #TCryptoRandom ifPresent:[:tc|  tc gatherEntropy: seed].
			each randomStream key: seed.
		].
	].
	root := self new.
	root borderWidth: 0.
	root newProject: worldMaker.
	root openInWorld.
	].! !

!CProjectMorph class methodsFor: 'as yet unclassified' stamp: 'ar 5/26/2005 23:15'!
serviceLoadProject

	^ SimpleServiceEntry 
			provider: self 
			label: 'load project'
			selector: #openProjectNamed:
			description: 'open project'! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 5/24/2005 17:46'!
aboutToEnterWorld
	"Morphic is about to enter the World that I belong to."


	self resumeTweakWorld! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 5/24/2005 17:46'!
aboutToLeaveWorld
	"Morphic is about to leave the World that I belong to."

	self suspendTweakWorld! !

!CProjectMorph methodsFor: 'menu' stamp: 'ar 6/20/2005 18:38'!
addModelItemsToWindowMenu: aMenu
	"aMenu is being constructed to be presented to the user in response to the user's pressing on the menu widget in the title bar of a morphic window.  Here, the model is given the opportunity to add any model-specific items to the menu, whose default target is the SystemWindow itself."
	aMenu addLine.
	aMenu add: 'Open Project' target: self action: #openProject.
	aMenu add: 'Save Project' target: self action: #saveProject.
	aMenu add: 'Save Project As' target: self action: #saveProjectAs.
	aMenu addLine.
	aMenu add: 'Export Image' target: self action: #exportImage.
! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/26/2005 23:02'!
adoptPaneColor: aColor
	project ifNil:[^super adoptPaneColor: aColor].
	self signalChanged: #windowColorChanged from: nil to: aColor.
	project world color: aColor veryMuchLighter.! !

!CProjectMorph methodsFor: 'drawing' stamp: 'ar 6/6/2007 23:41'!
areasRemainingToFill: aRectangle
	noRedraw ifTrue:[^Array with: aRectangle].
	(color isColor and: [color isTranslucent]) ifTrue: [^ Array with: aRectangle].
	self wantsRoundedCorners
		ifTrue: [^ aRectangle areasOutside: self boundsWithinCorners]
		ifFalse: [^ aRectangle areasOutside: self bounds]! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 2/9/2006 11:23'!
closeProject
	"Close this project"
	island ifNil:[^self].
	island project signal: #shutDownProject with: true.
	island scheduler run.
	island project destroy.
	island scheduler shutDown.
	island := project := nil.
! !

!CProjectMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 21:02'!
convertDropEvent: anEvent
	^CDropFilesEvent new 
			setPosition: anEvent position 
			contents: anEvent contents
			hand: nil
! !

!CProjectMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 21:02'!
convertKeyboardEvent: anEvent
	^CKeyboardEvent new
		setType: anEvent type
		buttons: anEvent buttons
		position: anEvent position
		keyValue: anEvent keyValue
		hand: nil
		stamp: anEvent timeStamp! !

!CProjectMorph methodsFor: 'event handling' stamp: 'ar 8/28/2005 21:04'!
convertMorphicEvent: anEvent
	"Convert a MorphicEvent"
	anEvent isMorphicEvent ifFalse:[^anEvent].
	anEvent isMouse ifTrue:[^self convertMouseEvent: anEvent].
	anEvent isKeyboard ifTrue:[^self convertKeyboardEvent: anEvent].
	anEvent type == #dropFilesEvent ifTrue:[^self convertDropEvent: anEvent].
	^nil! !

!CProjectMorph methodsFor: 'event handling' stamp: 'jdm 4/1/2008 11:20'!
convertMouseEvent: anEvent
	anEvent class == MouseMoveEvent ifTrue:[
		^CMouseMoveEvent new 
			setType: anEvent type 
			startPoint: anEvent startPoint
			endPoint: anEvent endPoint
			trail: anEvent trail
			buttons: anEvent buttons
			hand: nil
			stamp: anEvent timeStamp.
	].
	anEvent class == MouseButtonEvent ifTrue:[
		^CMouseButtonEvent new 
			setType: anEvent type 
			position: anEvent position
			which: anEvent whichButton
			buttons: anEvent buttons
			nrClicks: anEvent nrClicks
			hand: nil
			stamp: anEvent timeStamp.
	].
	^nil! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/24/2005 17:45'!
defaultBackgroundColor
	^CWidgetLibrary default baseColor! !

!CProjectMorph methodsFor: 'drawing' stamp: 'ar 5/26/2005 23:08'!
draw
	"Redraw everything"
	| canvas |
	canvas := CTransformCanvas on: display.
	canvas fillRectangle: display boundingBox color: Color white.
	project world asPrimCostume localFullDrawOn: canvas.
	self changed.! !

!CProjectMorph methodsFor: 'drawing' stamp: 'ar 6/6/2007 23:33'!
drawOn: aCanvas
	noRedraw ifTrue:[^nil].
	aCanvas fillRectangle: self bounds fillStyle: Color white borderStyle: self borderStyle.
	aCanvas clipBy: self innerBounds during:[:cc|
		cc drawImage: display at: (self innerBounds origin).
	].
	self isStepping ifFalse:[
		aCanvas frameAndFillRectangle: (bounds center - (100@25) extent: 200@50)
			fillColor: Color red borderWidth: 2 borderColor: Color black.
		aCanvas drawString: 'SCHEDULER STOPPED!!' at: bounds center - (60@15).
		aCanvas drawString: 'Click to restart' at: bounds center - (50@-5).
	].
! !

!CProjectMorph methodsFor: 'menu' stamp: 'ar 6/20/2005 18:41'!
exportImage
	| menu result fileName |
	menu := StandardFileMenu newFileMenu: (FileDirectory default) withPattern: '*.png'.
	result := menu startUpWithCaption: 'Enter Image Name...'.
	result ifNil:[^false].
	fileName := result directory fullNameFor: result name.
	(fileName asLowercase endsWith: '.png') ifFalse:[fileName := fileName,'.png'].
	PNGReadWriter putForm: project world asForm onFileNamed: fileName.! !

!CProjectMorph methodsFor: 'geometry' stamp: 'ar 5/26/2005 23:14'!
extent: newExtent
	super extent: newExtent.
	display ifNotNil:[display setExtent: newExtent depth: display depth].
	project ifNotNil:[project world extent: self innerBounds extent].
	self redraw.! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
fullScreenMode
	^owner ~~ window! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 6/24/2005 21:08'!
fullScreenMode: aBool
	aBool = self fullScreenMode ifTrue:[^self].
	aBool ifTrue:[
		self world addMorphFront: self.
		self borderWidth: 0.
		self topLeft: 0@0.
		self extent: DisplayScreen actualScreenSize.
	] ifFalse:[
		window addMorphBack: self.
		self borderWidth: 2.
	].! !

!CProjectMorph methodsFor: 'events' stamp: 'jdm 4/3/2008 10:16'!
handleEvent: anEvent
	| evt |

	(anEvent type == #mouseDown and:[anEvent redButtonChanged]) ifTrue:[
		self isStepping ifFalse:[
			self changed.
			^self startStepping]].
	lastMouseEvent ifNil:[lastMouseEvent := ActiveHand lastEvent].
	(#(mouseDown mouseUp mouseMove mouseEnter mouseLeave
	keyDown keyUp keystroke
	dropFilesEvent) includes: anEvent type) ifFalse:[^super handleEvent: anEvent].
	(anEvent isMouse) ifTrue:[
		anEvent hand keyboardFocus: self.
		anEvent isMouseDown ifTrue:[
			anEvent hand mouseFocus: self.
			self comeToFront].
		anEvent isMouseUp ifTrue:[
			anEvent hand releaseMouseFocus: self.
		].
	].
	"pass it into myWorld"
	evt := anEvent translatedBy: self innerBounds origin negated.
	(evt isMouse and:[evt isMouseMove and:[lastMouseEvent isMouse and: [evt buttons ~= lastMouseEvent buttons]]]) ifTrue:[
		"fix up event type"
		(lastMouseEvent mouseButtons bitClear: evt mouseButtons) > 0 ifTrue:[
			evt := CMouseButtonEvent new 
				setType: #mouseUp 
				position: evt position
				which: (lastMouseEvent buttons bitClear: evt buttons)
				buttons: evt buttons
				hand: evt hand
				stamp: evt timeStamp.
		].
		(evt mouseButtons bitClear: lastMouseEvent mouseButtons) > 0 ifTrue:[
			evt := CMouseButtonEvent new 
				setType: #mouseDown
				position: evt position
				which: (evt buttons bitClear: lastMouseEvent buttons)
				buttons: evt buttons
				nrClicks: 1
				hand: evt hand
				stamp: evt timeStamp.
		].
	].
	lastMouseEvent := anEvent clone.
	evt type == #mouseMove ifTrue:[
		evt setTrail: (evt trail collect:[:pt| pt - bounds origin]).
	].
	"evt printString displayAt: 0@0."
	self passEvent: evt.
	anEvent wasHandled: true.! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 5/24/2005 17:45'!
handlerForMouseDown: anEvent
	^self! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 5/24/2005 17:45'!
handlesMouseOver: evt
	^true! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/26/2005 20:05'!
initialExtent
	^window extent! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 6/6/2007 23:33'!
initialize
	super initialize.
	noRedraw := false.
	display := Form extent: 100@100 depth: Display depth.
	color := Color transparent.
	self extent: 800@600.
	self center: World center.! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/24/2005 17:45'!
intoWorld: aWorld
	"The receiver has just appeared in a new world."

	aWorld ifNil: [^ self].
	super intoWorld: aWorld.
	aWorld 
		when: #aboutToLeaveWorld send: #aboutToLeaveWorld to: self;
		when: #aboutToEnterWorld send: #aboutToEnterWorld to: self.
! !

!CProjectMorph methodsFor: 'stepping' stamp: 'ar 5/26/2005 23:08'!
invalidate
	"Redraw everything"
	| canvas |
	canvas := CTransformCanvas on: display.
	canvas fillRectangle: display boundingBox color: Color white.
	project world asPrimCostume localFullDrawOn: canvas.
	self changed.! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
label
	window ifNil:[^''].
	^window label! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
label: aString
	window ifNil:[^nil].
	window setLabel: aString! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/24/2005 17:45'!
modelSleep
	"This is useful when we pop up a debugger"
	self isStepping ifFalse:[self changed].! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/24/2005 17:45'!
modelWakeUp
	"This is useful when we pop up a debugger"
	self isStepping ifFalse:[self changed].! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
morphicLayerNumber
	self fullScreenMode ifTrue:[^20] ifFalse:[^super morphicLayerNumber].! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 5/24/2005 17:45'!
mouseEnter: evt
	evt hand newKeyboardFocus: self.! !

!CProjectMorph methodsFor: 'private' stamp: 'ar 5/26/2005 14:14'!
myEventMap
	^myEventMap! !

!CProjectMorph methodsFor: 'private' stamp: 'ar 5/26/2005 14:15'!
myEventMap: aMap
	myEventMap := aMap! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 9/30/2009 16:54'!
newProject: worldMaker
	| world |
	"Allow unsynchronized modification of the Tweak World while setting it up."
	[island := CProjectIsland new.
	island scheduler: (island new: ScriptScheduler).
	island project: (project := island new: CProject2 defaultProject).
	world := island new: worldMaker.
	((worldMaker includesBehavior: CPanel) and:[world window == nil]) ifTrue:[
		world window: self.
	].
	(worldMaker canUnderstand: #projectMorph:) ifTrue:[
		world projectMorph: self.
	].
	project newProjectWith: world] on: CUnsynchronizedModification do:[:ex| ex resume].
	self setExtentFromWorld.! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 6/6/2007 23:34'!
noRedraw
	"Answer whether we want to suppress drawing (for app-launches)"
	^noRedraw! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 6/6/2007 23:34'!
noRedraw: aBool
	"Indicate whether we want to suppress drawing (for app-launches)"
	noRedraw := aBool! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 5/26/2005 22:24'!
openProject
	| menu result |
	menu := StandardFileMenu oldFileMenu: (FileDirectory default) withPattern: '*.prj'.
	result := menu startUpWithCaption: 'Select Project to open...'.
	result ifNil: [ ^self ].
	self openProjectNamed: (result directory fullNameFor: result name)! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 5/28/2005 18:11'!
openProjectNamed: fullName
	| file |
	self profile:[
		file := FileStream readOnlyFileNamed: fullName.
		[self readProjectFrom: file] ensure:[file close].
	].
! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/24/2005 17:45'!
outOfWorld: aWorld
	"The receiver has just left aWorld."

	aWorld ifNil: [^ self].
	aWorld removeActionsWithReceiver: self.
	self suspendTweakWorld.
	super outOfWorld: aWorld..! !

!CProjectMorph methodsFor: 'events' stamp: 'ar 10/11/2007 14:09'!
passEvent: anEvent
	| evt |
	project ifNil:[^nil].
	(evt := self convertMorphicEvent: anEvent) ifNil:[^nil].
	"We are not interested in enter/over/leave events because they are notifications within a framework and Tweak implements its own policies regarding enter/leave. Filter them out to avoid confusion with the events generated by Tweak."
	(evt type == #mouseOver
		or:[evt type == #mouseEnter
		or:[evt type == #mouseLeave]]) ifTrue:[^self].
	evt := island shallowCopy: evt.
	evt := evt transformedBy: project world transform.
	project eventQueue nextPut: evt.! !

!CProjectMorph methodsFor: 'private' stamp: 'ar 5/26/2005 22:11'!
profile: aBlock
	^aBlock value! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:53'!
project
	^project! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 6/6/2007 23:38'!
projectWorld
	^project world! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 5/28/2005 21:58'!
readProjectFrom: file
	self closeProject.
	Cursor read showWhile:[
		island := CProjectReader new readProjectFrom: file.
	].
	project := island project.
	project signal: #startUpProject with: true.
	project fileName: file fullName.
	self label: file localName.
	((project world isKindOf: CPanel) and:[project world window == nil]) ifTrue:[
		project world window: self.
	].
	self setExtentFromWorld.! !

!CProjectMorph methodsFor: 'drawing' stamp: 'ar 5/26/2005 23:14'!
redraw
	"Redraw everything"
	| canvas |
	project ifNil:[^self].
	canvas := CTransformCanvas on: display.
	canvas fillRectangle: display boundingBox color: Color white.
	project world asPrimCostume localFullDrawOn: canvas.! !

!CProjectMorph methodsFor: 'private' stamp: 'ar 5/26/2005 22:02'!
resumeTweakWorld
	"Notify our world that it is being resumed"
	project ifNil:[^self].
	project world signal: #resume.
	island scheduler run.! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 5/28/2005 18:11'!
saveProject
	"Save the current project"
	| fileName file |
	project fileName ifNil:[^self saveProjectAs].
	fileName := project fileName.
	self profile:[
		file := FileStream forceNewFileNamed: fileName.
		[self saveProjectOn: file] ensure:[file close].
	].
	"For testing - reload the project immediately"
	self closeProject.
	self profile:[
		file := FileStream readOnlyFileNamed: fileName.
		[self readProjectFrom: file] ensure:[file close].
	].
	^true! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 5/26/2005 21:56'!
saveProjectAs
	"Save the current project"
	| menu result |
	menu := StandardFileMenu newFileMenu: (FileDirectory default) withPattern: '*.prj'.
	result := menu startUpWithCaption: 'Enter Project Name...'.
	result ifNil:[^false].
	project fileName: (result directory fullNameFor: result name).
	self label: result name.
	^self saveProject.! !

!CProjectMorph methodsFor: 'project' stamp: 'ar 6/18/2005 14:15'!
saveProjectOn: file
	| saveIsland writer |
	island ifNil:[^nil].

	writer := CProjectWriter new.

	((project world isKindOf: CPanel) and:[project world window == self]) ifTrue:[
		project world window: nil.
		island scheduler run.
	].

	"Dump the island while saving so we get no extra references"
	saveIsland := island.
	island := project := nil.

	"Prepare the project for saving"
	saveIsland project signal: #shutDownProject with: false.
	saveIsland scheduler run.

	"Checkpoint project"
	Cursor wait showWhile:[
		[writer snapshot: saveIsland] ensure:[
			"Start the project again"
			saveIsland project signal: #startUpProject with: false.
			island := saveIsland.
			project := saveIsland project.
		].
	].
	writer failed ifTrue:[^self inform: 'Checkpoint failed'].
	Cursor write showWhile:[writer saveProjectOn: file].! !

!CProjectMorph methodsFor: 'geometry' stamp: 'ar 5/26/2005 22:50'!
setExtentFromWorld
	window 
		ifNil:[self extent: project world extent]
		ifNotNil:[window extent: (project world extent + window fullBounds extent - self innerBounds extent)].! !

!CProjectMorph methodsFor: 'events' stamp: 'sn 9/10/2013 16:04'!
setHardwareCursor
	"Update the mouse cursor"
	| myHand cc |
	project ifNil:[^self].
	myHand := project primaryHand.
	cc := myHand mouseCursor.
	cc isCursor ifFalse:[cc := Cursor blank].
	cc == Cursor currentCursor ifFalse:[Cursor currentCursor: cc].! !

!CProjectMorph methodsFor: 'stepping' stamp: 'bgf 3/25/2009 09:58'!
step
	| script canvas damageList offset sz |
	project ifNil:[^self].
	"Check for a display change"
	self fullScreenMode ifTrue:[
		sz := DisplayScreen actualScreenSize.
		sz = self extent ifFalse:[
			DisplayScreen checkForNewScreenSize.
			self bounds: Display boundingBox].
	].
	"Have the project run its next cycle"
	script := project startScript: #runOneCycle.
	script hand: project primaryHand.
	false ifTrue: [
		"Useful debugging aid for nested world step problems."
		Display reverse: (0@0 extent: 10@10).
	].
	island scheduler run.
	offset := bounds origin + borderWidth.
	canvas := CTransformCanvas on: display.
	damageList := project drawOn: canvas.
	damageList do:[:each| self invalidRect: (each translateBy: offset)].
	(window isNil or: [window isActive]) ifTrue: [self setHardwareCursor].! !

!CProjectMorph methodsFor: 'stepping' stamp: 'ar 5/24/2005 17:45'!
stepAt: millisecondClockValue in: aWindow
	"do nothing"! !

!CProjectMorph methodsFor: 'stepping' stamp: 'ar 5/24/2005 17:45'!
stepTime
	^20! !

!CProjectMorph methodsFor: 'private' stamp: 'ar 5/26/2005 22:02'!
suspendTweakWorld
	"Notify our world that it is being suspended."
	project ifNil:[^self].
	project world signal: #suspend.
	island scheduler run.! !

!CProjectMorph methodsFor: 'stepping' stamp: 'ar 5/24/2005 17:46'!
wantsSteps
	^true! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
window
	^window! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/24/2005 17:45'!
window: aWindow
	window := aWindow! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 8/22/2006 15:16'!
windowColor
	^window ifNil:[self color] ifNotNil:[window paneColor]! !

!CProjectMorph methodsFor: 'accessing' stamp: 'ar 5/26/2005 14:17'!
windowColor: aColor
	self signalChanged: #windowColorChanged from: self windowColor to: aColor.
	window paneColor: aColor.
	project world color: aColor veryMuchLighter.! !

!CProjectMorph methodsFor: 'initialize' stamp: 'ar 5/30/2005 15:44'!
windowIsClosing
	self closeProject.! !

CPlayer subclass: #CProjectNavigator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectNavigator commentStamp: '<historical>' prior: 0!
A simple project navigator!

CProjectNavigator defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="projectShell" toSet="projectShell:" changeEvent="projectShellChanged">projectShell</field>
	<field toGet="projectList" toSet="projectList:" changeEvent="projectListChanged">projectList</field>
	<field toGet="prevButton" toSet="prevButton:" changeEvent="prevButtonChanged">prevButton</field>
	<field toGet="nextButton" toSet="nextButton:" changeEvent="nextButtonChanged">nextButton</field>
	<field toGet="menuButton" toSet="menuButton:" changeEvent="menuButtonChanged">menuButton</field>
	<field toGet="anchorPoint" toSet="anchorPoint:" changeEvent="anchorPointChanged">anchorPoint</field>
</fields>'!

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 17:18'!
anchorPoint
	"Answer the anchorPoint of the receiver"
	<bewareOf: #anchorPointChanged>
	^self propertyValueAt: #anchorPoint! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 17:18'!
anchorPoint: aValue
	"Modify the receiver's anchorPoint"
	^self propertyValueAt: #anchorPoint put: aValue with: #anchorPointChanged! !

!CProjectNavigator methodsFor: 'initialize' stamp: 'ar 8/5/2005 17:59'!
initialize
	super initialize.
	self define: #prevButton as: CImageButton new.
	self define: #nextButton as: CImageButton new.
	self define: #menuButton as: CEllipsePlayer new.
	self projectList: CCollection new.
	anchorPoint := #bottomRight.! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 14:16'!
menuButton
	"Answer the menuButton of the receiver"
	<bewareOf: #menuButtonChanged>
	^self propertyValueAt: #menuButton! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 14:16'!
menuButton: aValue
	"Modify the receiver's menuButton"
	^self propertyValueAt: #menuButton put: aValue with: #menuButtonChanged! !

!CProjectNavigator methodsFor: 'menu' stamp: 'ar 8/5/2005 17:52'!
navMenu
	| menu item |
	menu := CMenu new.
	menu add: 'close navigator' target: self action: #destroy.
	menu addSeparator.
	item := menu add: 'top left' target: self action: #toggleAnchor: argument: #topLeft.
	item checked: anchorPoint == #topLeft.
	item := menu add: 'top right' target: self action: #toggleAnchor: argument: #topRight.
	item checked: anchorPoint == #topRight.
	item := menu add: 'bottom left' target: self action: #toggleAnchor: argument: #bottomLeft.
	item checked: anchorPoint == #bottomLeft.
	item := menu add: 'bottom right' target: self action: #toggleAnchor: argument: #bottomRight.
	item checked: anchorPoint == #bottomRight.
	^menu! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
nextButton
	"Answer the nextButton of the receiver"
	<bewareOf: #nextButtonChanged>
	^self propertyValueAt: #nextButton! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
nextButton: aValue
	"Modify the receiver's nextButton"
	^self propertyValueAt: #nextButton put: aValue with: #nextButtonChanged! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 17:44'!
onContainerLayoutChanged
	| box cc delta |
	<on: containerLayoutChanged>
	cc := container ifNil:[^nil].
	anchorPoint ifNil:[^nil].
	box := (0@0 extent: cc extent) insetBy: cc borderWidth.
	delta := (box perform: anchorPoint) - (self bounds perform: anchorPoint).
	delta isZero ifTrue:[^nil].
	position := position + delta.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 17:53'!
onDragMove
	"Drag the receiver but do not drop it when the move completes"
	| nextPos moved cursor delta wasLocked shadow offset |
	container ifNil:[^self].
	anchorPoint := nil.
	wasLocked := self locked.
	self locked: true.

	cursor := self mouseCursor.
	self mouseCursor: (cursor ifNil:[Cursor normal]) asForm.
	self hand cursorFocus: self.

	offset := self cursorPoint.
	shadow := CDropShadowHighlight new.
	shadow name: #dragMoveShadow.
	shadow offset: 6@8.

	moved := false.
	self hand forEach: #mouseMove do:[
		nextPos := self cursorPoint.
		delta := nextPos - offset.
		moved ifFalse:[
			self addHighlight: shadow.
			moved := true.
		].
		self transform: ((COffsetTransform withOffset: delta) transformedBy: self transform).
	] until: #mouseUp.
	self mouseCursor: cursor.
	self hand releaseMouseFocus: self.
	self removeHighlight: #dragMoveShadow.
	self locked: wasLocked.
	self hand releaseCursorFocus: self.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 14:20'!
onMouseClick
	<on: mouseClick in: menuButton>
	self navMenu open! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 14:52'!
onMouseDown
	<on: mouseDown>
	self onDragMove.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 14:52'!
onMouseDrag
	<on: mouseDrag in: menuButton>
	self onDragMove.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 13:56'!
onNextButtonFire
	<on: fire in: nextButton>
	projectList atEnd ifFalse:[projectList next].! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 18:03'!
onNextButtonStateChanged
	<on: stateChanged in: nextButton>
	self updateButtonState: nextButton.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 13:57'!
onPrevButtonFire
	<on: fire in: prevButton>
	projectList atStart ifFalse:[projectList prev].! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 18:03'!
onPrevButtonStateChanged
	<on: stateChanged in: prevButton>
	self updateButtonState: prevButton.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 17:57'!
onProjectListChanged
	<on: projectListChanged>
	self updateButtons.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 13:59'!
onProjectListCursorChanged
	| projectIsland |
	<on: cursorChanged in: projectList>
	projectShell ifNotNil:[projectIsland := projectList selection].
	projectIsland ifNotNil:[
		projectShell myIsland: projectIsland selection.
		projectShell myProject: projectIsland project.
	].
	self updateButtons.! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
prevButton
	"Answer the prevButton of the receiver"
	<bewareOf: #prevButtonChanged>
	^self propertyValueAt: #prevButton! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
prevButton: aValue
	"Modify the receiver's prevButton"
	^self propertyValueAt: #prevButton put: aValue with: #prevButtonChanged! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
projectList
	"Answer the projectList of the receiver"
	<bewareOf: #projectListChanged>
	^self propertyValueAt: #projectList! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:34'!
projectList: aValue
	"Modify the receiver's projectList"
	^self propertyValueAt: #projectList put: aValue with: #projectListChanged! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:56'!
projectShell
	"Answer the projectShell of the receiver"
	<bewareOf: #projectShellChanged>
	^self propertyValueAt: #projectShell! !

!CProjectNavigator methodsFor: 'accessing' stamp: 'ar 8/5/2005 13:56'!
projectShell: aValue
	"Modify the receiver's projectShell"
	^self propertyValueAt: #projectShell put: aValue with: #projectShellChanged! !

!CProjectNavigator methodsFor: 'initialize' stamp: 'ar 8/5/2005 17:55'!
setupCostume
	| pt |
	super setupCostume.
	self roundCorners: true.
	color := CWidgetLibrary default baseColor alpha: 0.5.
	pt := 4@4.
	prevButton topLeft: pt.
	prevButton extent: 52@39.
	prevButton color: Color veryLightGray.
	self addLast: prevButton.
	nextButton topLeft: prevButton topRight + (pt x @ 0).
	nextButton extent: 52@39.
	nextButton color: Color veryLightGray.
	self addLast: nextButton.
	self extent: nextButton bottomRight + pt.
	menuButton color: CWidgetLibrary default baseColor.
	menuButton extent: 16@16.
	menuButton borderWidth: 1.
	menuButton position: self extent // 2.
	menuButton balloonHelp: 'Click to get a menu. Drag to move the navigator'.
	self add: menuButton.! !

!CProjectNavigator methodsFor: 'menu' stamp: 'ar 8/5/2005 17:53'!
toggleAnchor: aSymbol
	"Toggle anchor point"
	anchorPoint == aSymbol
		ifTrue:[^anchorPoint := nil].
	anchorPoint := aSymbol.
	self onContainerLayoutChanged.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 18:04'!
updateButtonState: button
	button state == #over 
		ifTrue:[^button borderWidth: 2].
	button state == #pressed 
		ifTrue:[^button borderWidth: 4].
	button borderWidth: 0.! !

!CProjectNavigator methodsFor: 'events' stamp: 'ar 8/5/2005 18:05'!
updateButtons
	"Update my button display"
	| form thumb |
	form := Form extent: 52@39 depth: 32.
	form fillColor: Color lightGray.
	projectList cursor > 1 ifTrue:[
		form fillColor: Color veryLightGray.
		thumb := (projectList at: projectList cursor - 1) project world asPrimCostume asFormExtent: form extent.
		thumb displayOn: form at: 0@0 rule: Form paint.
		(form getCanvas asBalloonCanvas)
			drawPolygon: {
				2@(form height // 2).
				18@(form height // 8).
				18@(form height * 7 // 8).
			} color: CWidgetLibrary default baseColor borderWidth: 0 borderColor: Color gray.
		prevButton enable.
	] ifFalse:[prevButton disable].
	prevButton graphicForm: form.

	form := Form extent: 52@39 depth: 32.
	form fillColor: Color lightGray.
	projectList cursor < projectList size ifTrue:[
		form fillColor: Color veryLightGray.
		thumb := (projectList at: projectList cursor + 1) project world asPrimCostume asFormExtent: form extent.
		thumb displayOn: form at: 0@0 rule: Form paint.
		(form getCanvas asBalloonCanvas)
			drawPolygon: {
				form width-2@(form height // 2).
				form width-18@(form height // 8).
				form width-18@(form height * 7 // 8).
			} color: CWidgetLibrary default baseColor borderWidth: 0 borderColor: Color gray.
		nextButton enable.
	] ifFalse:[nextButton disable].
	nextButton graphicForm: form.
! !

CPlayer subclass: #CProjectShell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectShell commentStamp: '<historical>' prior: 0!
A project holder encapsulates a project inside CProjectBuilder.!

CProjectShell defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="myIsland" toSet="myIsland:" changeEvent="myIslandChanged">myIsland</field>
	<field toGet="myProject" toSet="myProject:" changeEvent="myProjectChanged">myProject</field>
	<field toGet="tool" toSet="tool:" changeEvent="toolChanged">tool</field>
	<field toGet="halo" toSet="halo:" changeEvent="haloChanged">halo</field>
</fields>'!

!CProjectShell methodsFor: 'project' stamp: 'ar 2/9/2006 11:21'!
closeProject
	myIsland ifNil:[^self].
	myIsland project signal: #shutDownProject with: true.
	myIsland scheduler run.
	myIsland project destroy.
	myIsland scheduler shutDown.
	myIsland := nil.
! !

!CProjectShell methodsFor: 'drawing' stamp: 'ar 6/28/2005 19:24'!
drawOn: aCanvas in: drawingBounds
	"Draw background and border"
	super drawOn: aCanvas in: drawingBounds.
	myProject ifNil:[
		aCanvas fillRectangle: (drawingBounds insetBy: borderWidth) color: Color gray.
	] ifNotNil:[
		myProject drawOn: aCanvas in: drawingBounds
	].! !

!CProjectShell methodsFor: 'project' stamp: 'ar 5/28/2005 18:12'!
fileName
	^myProject fileName! !

!CProjectShell methodsFor: 'project' stamp: 'ar 5/28/2005 18:12'!
fileName: aString
	myProject fileName: aString.! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/20/2005 15:20'!
halo
	"Answer the halo of the receiver"
	<bewareOf: #haloChanged>
	^self propertyValueAt: #halo! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/20/2005 15:20'!
halo: aValue
	"Modify the receiver's halo"
	^self propertyValueAt: #halo put: aValue with: #haloChanged! !

!CProjectShell methodsFor: 'initialize' stamp: 'ar 2/7/2005 11:20'!
initialize
	super initialize.
	self userDraw := true.
! !

!CProjectShell methodsFor: 'initialize' stamp: 'ar 5/26/2005 00:49'!
makeNew: dragMaker in: box
	"Create a new instance from aMaker and stuff it into the project. Assign it the given bounds. This is simple and incorrect for now but the correct idea is to localize these changes here."
	| obj |
	myProject ifNil:[^self beep].
	obj := myIsland send: dragMaker msg to: dragMaker maker.
	obj bounds: box.
	myProject world add: obj.! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 6/28/2005 18:27'!
mouseCursor
	| cc myHand |
	myProject ifNil:[^nil].
	myHand := myProject primaryHand.
	myHand showHardwareCursor ifTrue:[
		cc := myHand mouseCursor.
		cc isCursor ifTrue:[^cc].
	].
	^Cursor blank! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:43'!
myIsland
	"Answer the myIsland of the receiver"
	<bewareOf: #myIslandChanged>
	^self propertyValueAt: #myIsland! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:43'!
myIsland: aValue
	"Modify the receiver's myIsland"
	^self propertyValueAt: #myIsland put: aValue with: #myIslandChanged! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:43'!
myProject
	"Answer the myProject of the receiver"
	<bewareOf: #myProjectChanged>
	^self propertyValueAt: #myProject! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/26/2005 00:43'!
myProject: aValue
	"Modify the receiver's myProject"
	^self propertyValueAt: #myProject put: aValue with: #myProjectChanged! !

!CProjectShell methodsFor: 'initialize' stamp: 'ar 7/8/2007 13:35'!
newProject
	myIsland := CProjectIsland new.
	myIsland scheduler: (myIsland new: ScriptScheduler).
	myIsland project: (myProject := myIsland new: CProject2 defaultProject).
	myProject newProject.! !

!CProjectShell methodsFor: 'tool' stamp: 'ar 2/7/2005 16:31'!
newTool: aTool
	tool ifNotNil:[self releaseTool: tool].
	self attachBehavior: aTool.
	tool := aTool.
	mouseCursor := aTool mouseCursor.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onBlueButtonDown
	<on: blueButtonDown>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onBlueButtonUp
	<on: blueButtonUp>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 6/26/2005 15:01'!
onExtentChanged
	<on: extentChanged>
	myProject ifNil:[^nil].
	myProject world ifNil:[^nil].
	myProject world extent: self extent.! !

!CProjectShell methodsFor: 'private' stamp: 'ar 5/30/2005 13:26'!
onFileIn
	| file |
	<on: fileIn>
	file := FileStream readOnlyFileNamed: 'test.prj'.
	[
		file binary.
		MessageTally spyOnScript:[
			1 to: 10 do:[:i|
				file position: 0.
				self readProjectFrom: file.
			].
		].
	] ensure:[file close].! !

!CProjectShell methodsFor: 'private' stamp: 'ar 5/30/2005 13:26'!
onFileOut
	| file |
	<on: fileOut>
	file := FileStream forceNewFileNamed: 'test.prj'.
	[
		file binary.
		MessageTally spyOnScript:[
			1 to: 10 do:[:i|
				file position: 0.
				self saveProjectOn: file.
			].
		].
	] ensure:[file close].! !

!CProjectShell methodsFor: 'events' stamp: 'ar 6/26/2005 15:02'!
onIslandChanged
	<on: myIslandChanged>
	self stopScript: #onShutDownProject:.
	self stopScript: #runProject.
	myProject := halo := nil.
	myIsland ifNotNil:[
		myProject := myIsland project.
		myProject world bounds: (0@0 extent: self extent).
		halo := myProject get: #halo.
		self startScript: #runProject.
		self startScript: #onShutDownProject: when:{Processor activeProject. #shutDownProject}.
	].
	self invalidate.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onKeyDown
	<on: keyDown>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onKeyStroke
	<on: keyStroke>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onKeyUp
	<on: keyUp>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 8/5/2005 13:03'!
onMouseDown
	<on: mouseDown>
	self project halo target: nil. "get rid of halo"
	hand keyboardFocus: self. "all keyboard input to me"
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 6/26/2005 15:02'!
onMouseEnter
	"Aquire keyboard focus by default but don't steal it"
	<on: mouseEnter>
	hand keyboardFocus ifNil:[hand keyboardFocus: self].
! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/11/2005 13:56'!
onMouseLeave
	<on: mouseLeave>
	hand releaseKeyboardFocus: self.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 7/14/2005 15:19'!
onMouseMove
	"Only pass mouseMove if we're the top-player under the hand"
	<on: mouseMove>
	hand playersUnder first asPrimCostume == self asPrimCostume
		ifTrue:[self passEvent].! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onMouseUp
	<on: mouseUp>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/10/2005 20:01'!
onProjectChanged
	<on: projectChanged>
	self invalidate.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 5/26/2005 01:00'!
onShutDownProject: quitting
	quitting ifTrue:[self closeProject].! !

!CProjectShell methodsFor: 'private' stamp: 'ar 5/30/2005 13:27'!
onSwapIn
	| file |
	<on: swapIn>
	file := FileStream readOnlyFileNamed: 'test.isg'.
	[
		file binary.
		MessageTally spyOnScript:[
			1 to: 10 do:[:i|
				file position: 0.
				self swapInProjectFrom: file.
			].
		].
	] ensure:[file close].! !

!CProjectShell methodsFor: 'private' stamp: 'ar 5/30/2005 13:27'!
onSwapOut
	| file |
	<on: swapOut>
	file := FileStream forceNewFileNamed: 'test.isg'.
	[
		file binary.
		MessageTally spyOnScript:[
			1 to: 10 do:[:i|
				file position: 0.
				self swapOutProjectOn: file.
			].
		].
	] ensure:[file close].! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onYellowButtonDown
	<on: yellowButtonDown>
	self passEvent.! !

!CProjectShell methodsFor: 'events' stamp: 'ar 5/21/2005 17:35'!
onYellowButtonUp
	<on: yellowButtonUp>
	self passEvent.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 5/21/2005 17:36'!
passEvent
	^self passEvent: (hand lastEvent clone resetHandlerFields setHand: nil).! !

!CProjectShell methodsFor: 'project' stamp: 'ar 6/26/2005 15:00'!
passEvent: event
	| evt |
	tool ifNotNil:[^nil]. "let tool handle it"
	myProject ifNil:[^nil].
	evt := event transformedBy: self worldToLocal.
	evt := myIsland shallowCopy: evt.
	myProject eventQueue nextPut: evt.! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 5/28/2005 19:12'!
projectWorld
	^myProject world! !

!CProjectShell methodsFor: 'project' stamp: 'ar 8/5/2005 12:14'!
readProjectFrom: file
	Cursor read showWhile:[
		myIsland := CProjectReader new readProjectFrom: file.
	].
	myProject := myIsland project.
	myProject signal: #startUpProject with: true.
	myIsland scheduler run.! !

!CProjectShell methodsFor: 'tool' stamp: 'ar 2/7/2005 16:30'!
releaseTool: aTool
	aTool == tool ifFalse:[^nil].
	self detachBehavior: aTool.
	tool := nil.
	mouseCursor := nil.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 6/26/2005 14:54'!
runOneProjectCycle
	| recorder list script myScheduler tfm |

	myIsland ifNil:[^self waitTick].

	myScheduler := myIsland scheduler.

	myProject world ifNotNil:[
		script := myProject startScript: #runOneCycle.
		script hand: myProject primaryHand.
	].

	[myScheduler hasActiveScripts] whileTrue:[
		myScheduler run.
		"Sync it so we can have event interactions between this and the outer scheduler"
		self syncScript.
	].

	myProject world ifNotNil:[
		"Post the invalidations"
		recorder := myProject world asPrimCostume localCache damageRecorder.
		list := recorder invalidRectsFullBounds: myProject world localBounds.
		tfm := myProject world transform.
		recorder reset.
		list do:[:rect| self invalidate: (tfm transformRect: rect)].
	].

	"Actually we should be waiting for more than one 
	tick but for now we're just going to play it simple."
	self waitTick.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 5/11/2005 15:24'!
runProject
	"Run this project"
	[true] whileTrue:[self runOneProjectCycle].! !

!CProjectShell methodsFor: 'project' stamp: 'ar 6/18/2005 14:15'!
saveProjectOn: file
	| saveIsland writer byteStream |
	myIsland ifNil:[^nil].

	writer := CProjectWriter new.
	byteStream := (ByteArray new: 100) writeStream.
	PNGReadWriter 
		putForm: (myIsland project world asPrimCostume asFormExtent: 128@128)
		onStream: byteStream.
	writer archive addString: byteStream contents as: 'project.png'.

	"Dump the island while saving so we get no extra references"
	saveIsland := myIsland.
	myIsland := nil.
	Processor activeIsland project killFrame: true.
	self waitTick.

	"Prepare the project for saving"
	saveIsland project signal: #shutDownProject with: false.
	saveIsland scheduler run.

	"Checkpoint project"
	Cursor wait showWhile:[
		[writer snapshot: saveIsland] ensure:[
			"Start the project again"
			saveIsland project signal: #startUpProject with: false.
			myIsland := saveIsland.
		].
	].
	writer failed ifNil:[^self inform: 'Checkpoint failed'].
	Cursor write showWhile:[writer saveProjectOn: file].! !

!CProjectShell methodsFor: 'initialize' stamp: 'ar 4/26/2005 17:20'!
setupCostume
	super setupCostume.
	self clipping: true.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 8/5/2005 12:14'!
swapInProjectFrom: file
	Cursor read showWhile:[
		myIsland := CProjectReader new swapInProjectFrom: file.
	].
	myProject := myIsland project.
	myProject signal: #startUpProject with: true.
	myIsland scheduler run.! !

!CProjectShell methodsFor: 'project' stamp: 'ar 6/18/2005 14:15'!
swapOutProjectOn: aStream
	"Swap out the project"
	| saveIsland writer |
	myIsland ifNil:[^nil].

	writer := CProjectWriter new.

	"Dump the island while saving so we get no extra references"
	saveIsland := myIsland.
	myIsland := nil.
	Processor activeIsland project killFrame: true.
	self waitTick.

	"Prepare the project for saving"
	saveIsland project signal: #shutDownProject with: false.
	saveIsland scheduler run.

	"Checkpoint project"
	Cursor wait showWhile:[
		[writer snapshot: saveIsland] ensure:[
			"Start the project again"
			saveIsland project signal: #startUpProject with: false.
			myIsland := saveIsland.
		].
	].
	writer failed ifNil:[^self inform: 'Checkpoint failed'].
	Cursor write showWhile:[writer swapOutProjectOn: aStream].! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 2/7/2005 11:19'!
tool
	"Answer the tool of the receiver"
	<bewareOf: #toolChanged>
	^self propertyValueAt: #tool! !

!CProjectShell methodsFor: 'accessing' stamp: 'ar 2/7/2005 11:19'!
tool: aValue
	"Modify the receiver's tool"
	^self propertyValueAt: #tool put: aValue with: #toolChanged! !

CPlayer subclass: #CProjectWorld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectWorld commentStamp: '<historical>' prior: 0!
I am a default project world.!

CProjectWorld defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="halo" toSet="halo:" changeEvent="haloChanged">halo</field>
</fields>'!

!CProjectWorld methodsFor: 'accessing' stamp: 'ar 5/24/2005 21:34'!
halo
	"Answer the halo of the receiver"
	<bewareOf: #haloChanged>
	^self propertyValueAt: #halo! !

!CProjectWorld methodsFor: 'accessing' stamp: 'ar 5/24/2005 21:34'!
halo: aValue
	"Modify the receiver's halo"
	^self propertyValueAt: #halo put: aValue with: #haloChanged! !

!CProjectWorld methodsFor: 'initialize' stamp: 'ar 5/24/2005 21:36'!
initialize
	super initialize.
	halo := CHalo new.
	halo userDraw: true.
	halo visible: false.
	self add: halo.! !

!CProjectWorld methodsFor: 'events' stamp: 'ar 5/24/2005 21:34'!
onBlueButtonDown
	<on: blueButtonDown>
	self halo onBlueButtonDown! !

!CProjectWorld methodsFor: 'events' stamp: 'ar 5/24/2005 21:33'!
onMouseDown
	| dragChild |
	<on: mouseDown>
	dragChild := self findDraggablePlayerAt: self cursorPoint.
	dragChild ifNotNil:[^dragChild onDrag].
	"clicks on desktop remove halo and keyboard focus"
	self hand keyboardFocus ifNotNil:[hand keyboardFocus: nil].
	self halo target ifNotNil:[self halo target: nil].! !

CCanvasCache subclass: #CProjectCache
	instanceVariableNames: 'damageRecorder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectCache commentStamp: 'ar 4/27/2005 15:56' prior: 0!
I am a pseudo cache used to record invalidations in top-level players (worlds).!

!CProjectCache methodsFor: 'accessing' stamp: 'ar 4/27/2005 15:54'!
damageRecorder
	^damageRecorder! !

!CProjectCache methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:55'!
drawCached: aBlock on: aCanvas in: bounds
	aBlock value: aCanvas! !

!CProjectCache methodsFor: 'initialize' stamp: 'ar 4/27/2005 15:55'!
initialize
	damageRecorder := CDamageRecorder new.
	damageRecorder doFullRepaint.! !

!CProjectCache methodsFor: 'drawing' stamp: 'ar 4/27/2005 15:55'!
invalidate: aRectangle
	damageRecorder invalidate: aRectangle.! !

IslandWriter subclass: #CProjectWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectWriter class methodsFor: 'class initialization' stamp: 'ar 6/16/2005 20:37'!
archiveVersionID
	^UUID fromString: '0e5a72be-c4f8-cd49-8a4d-fb7f891da0cd'! !

!CProjectWriter class methodsFor: 'class initialization' stamp: 'ar 6/16/2005 20:37'!
projectVersionID
	^UUID fromString: 'e0867ab2-b756-b74b-bd2b-500ec8fda08d'! !

!CProjectWriter methodsFor: 'private' stamp: 'ar 6/23/2005 16:41'!
rootsOf: anIsland
	roots := #().
	roots := roots, anIsland project scriptManager registeredUniClasses.
	roots := roots, anIsland scheduler scheduledScripts.
	roots := anIsland roots, roots.
	^roots copyWithout: nil! !

!CProjectWriter methodsFor: 'public' stamp: 'ar 6/16/2005 20:36'!
saveProjectOn: file
	| data |
	dataStream := IslandStream on: (ByteArray new: segment size * 5).
	dataStream nextPutAll: CProjectWriter projectVersionID.

	"Export data on dataStream"
	self export: imports.
	self exportImageSegment: segment.

	data := dataStream contents.
	archive 
		addDeflateString: data
		as: 'project.isg'.
	archive writeTo: file prepending: CProjectWriter archiveVersionID.
	^data! !

!CProjectWriter methodsFor: 'public' stamp: 'ar 6/16/2005 20:36'!
swapOutProjectOn: aStream

	self flag: #fixMe.	"We ought to be able to use the file"
	dataStream := IslandStream on: (ByteArray new: segment size * 5).
	dataStream nextPutAll: CProjectWriter projectVersionID.

	"Export data on dataStream"
	self export: imports.
	self exportImageSegment: segment.
	aStream nextPutAll: dataStream contents.
! !

!CProjectWriter methodsFor: 'private' stamp: 'ar 6/23/2005 16:38'!
validateRoots: roots
	"Overridden to allow uniclasses to get through."
	self flag: #fixMe. "uniclasses should be proper passByProxy: objects but for now all behaviors are passByIdentity:"
	roots do:[:obj|
		((IslandArgumentCopier isPassByIdentity: obj) and:[
			(obj isBehavior and:[obj isUniClass]) not])
				ifTrue:[self error:'Cannot have passByIdentity: roots'].
	].
! !

Island subclass: #CProjectIsland
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Projects'!

!CProjectIsland commentStamp: '<historical>' prior: 0!
An Island specifically used for projects.!

!CProjectIsland class methodsFor: 'utilities' stamp: 'ar 7/8/2007 13:35'!
newProject: worldMaker
	"Create a new project island with the given worldMaker"
	| island |
	island := self new.
	island scheduler: (island new: ScriptScheduler).
	island project: (island new: CProject2 defaultProject).
	island project newProject: worldMaker.
	island scheduler run.
	^island! !

!CProjectIsland methodsFor: 'initialize' stamp: 'ar 5/30/2005 21:03'!
initialize
	super initialize.
	self project: CProject2 new. "temporarily"! !

!ProcessorScheduler methodsFor: '*Tweak-Projects' stamp: 'ar 5/26/2005 18:17'!
activeProject
	"Answer the currently active Project."
	^self activeIsland project! !
CTweakProjectInMorphic initialize!
CProjectMorph initialize!
